@@ -1,9 +1,9 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 /*
  * Copyright (C) 2013 Red Hat, Inc.
  */
 
-#include "nm-default.h"
+#include "libnm-core/nm-default-libnm-core.h"
 
 #include "nm-setting-dcb.h"
 
@@ -893,7 +893,7 @@ nm_setting_dcb_init(NMSettingDcb *self)
 NMSetting *
 nm_setting_dcb_new(void)
 {
-    return (NMSetting *) g_object_new(NM_TYPE_SETTING_DCB, NULL);
+    return g_object_new(NM_TYPE_SETTING_DCB, NULL);
 }
 
 static void
