@@ -1,4 +1,4 @@
-// SPDX-License-Identifier: LGPL-2.1+
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 /*
  * Copyright (C) 2007 - 2014 Red Hat, Inc.
  */
@@ -6,20 +6,25 @@
 #ifndef __NM_SETTING_IP6_CONFIG_H__
 #define __NM_SETTING_IP6_CONFIG_H__
 
-#if !defined (__NETWORKMANAGER_H_INSIDE__) && !defined (NETWORKMANAGER_COMPILATION)
-#error "Only <NetworkManager.h> can be included directly."
+#if !defined(__NETWORKMANAGER_H_INSIDE__) && !defined(NETWORKMANAGER_COMPILATION)
+    #error "Only <NetworkManager.h> can be included directly."
 #endif
 
 #include "nm-setting-ip-config.h"
 
 G_BEGIN_DECLS
 
-#define NM_TYPE_SETTING_IP6_CONFIG            (nm_setting_ip6_config_get_type ())
-#define NM_SETTING_IP6_CONFIG(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NM_TYPE_SETTING_IP6_CONFIG, NMSettingIP6Config))
-#define NM_SETTING_IP6_CONFIG_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), NM_TYPE_SETTING_IP6CONFIG, NMSettingIP6ConfigClass))
-#define NM_IS_SETTING_IP6_CONFIG(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NM_TYPE_SETTING_IP6_CONFIG))
-#define NM_IS_SETTING_IP6_CONFIG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), NM_TYPE_SETTING_IP6_CONFIG))
-#define NM_SETTING_IP6_CONFIG_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), NM_TYPE_SETTING_IP6_CONFIG, NMSettingIP6ConfigClass))
+#define NM_TYPE_SETTING_IP6_CONFIG (nm_setting_ip6_config_get_type())
+#define NM_SETTING_IP6_CONFIG(obj) \
+    (G_TYPE_CHECK_INSTANCE_CAST((obj), NM_TYPE_SETTING_IP6_CONFIG, NMSettingIP6Config))
+#define NM_SETTING_IP6_CONFIG_CLASS(klass) \
+    (G_TYPE_CHECK_CLASS_CAST((klass), NM_TYPE_SETTING_IP6CONFIG, NMSettingIP6ConfigClass))
+#define NM_IS_SETTING_IP6_CONFIG(obj) \
+    (G_TYPE_CHECK_INSTANCE_TYPE((obj), NM_TYPE_SETTING_IP6_CONFIG))
+#define NM_IS_SETTING_IP6_CONFIG_CLASS(klass) \
+    (G_TYPE_CHECK_CLASS_TYPE((klass), NM_TYPE_SETTING_IP6_CONFIG))
+#define NM_SETTING_IP6_CONFIG_GET_CLASS(obj) \
+    (G_TYPE_INSTANCE_GET_CLASS((obj), NM_TYPE_SETTING_IP6_CONFIG, NMSettingIP6ConfigClass))
 
 #define NM_SETTING_IP6_CONFIG_SETTING_NAME "ipv6"
 
@@ -39,7 +44,7 @@ G_BEGIN_DECLS
  * IPv6 is not required or is handled by some other mechanism, and NetworkManager
  * should not configure IPv6 for this connection.
  */
-#define NM_SETTING_IP6_CONFIG_METHOD_IGNORE     "ignore"
+#define NM_SETTING_IP6_CONFIG_METHOD_IGNORE "ignore"
 
 /**
  * NM_SETTING_IP6_CONFIG_METHOD_AUTO:
@@ -48,7 +53,7 @@ G_BEGIN_DECLS
  * for the hardware interface, ie router advertisements, DHCP, or PPP or some
  * other device-specific manner.
  */
-#define NM_SETTING_IP6_CONFIG_METHOD_AUTO       "auto"
+#define NM_SETTING_IP6_CONFIG_METHOD_AUTO "auto"
 
 /**
  * NM_SETTING_IP6_CONFIG_METHOD_DHCP:
@@ -56,7 +61,7 @@ G_BEGIN_DECLS
  * IPv6 configuration should be automatically determined via DHCPv6 only and
  * router advertisements should be ignored.
  */
-#define NM_SETTING_IP6_CONFIG_METHOD_DHCP       "dhcp"
+#define NM_SETTING_IP6_CONFIG_METHOD_DHCP "dhcp"
 
 /**
  * NM_SETTING_IP6_CONFIG_METHOD_LINK_LOCAL:
@@ -72,7 +77,7 @@ G_BEGIN_DECLS
  * All necessary IPv6 configuration (addresses, prefix, DNS, etc) is specified
  * in the setting's properties.
  */
-#define NM_SETTING_IP6_CONFIG_METHOD_MANUAL     "manual"
+#define NM_SETTING_IP6_CONFIG_METHOD_MANUAL "manual"
 
 /**
  * NM_SETTING_IP6_CONFIG_METHOD_SHARED:
@@ -85,7 +90,7 @@ G_BEGIN_DECLS
  * devices to connect through that interface to the default network. (not yet
  * supported for IPv6)
  */
-#define NM_SETTING_IP6_CONFIG_METHOD_SHARED     "shared"
+#define NM_SETTING_IP6_CONFIG_METHOD_SHARED "shared"
 
 /**
  * NM_SETTING_IP6_CONFIG_METHOD_DISABLED:
@@ -94,7 +99,7 @@ G_BEGIN_DECLS
  *
  * Since: 1.20
  */
-#define NM_SETTING_IP6_CONFIG_METHOD_DISABLED   "disabled"
+#define NM_SETTING_IP6_CONFIG_METHOD_DISABLED "disabled"
 
 /**
  * NMSettingIP6ConfigPrivacy:
@@ -109,10 +114,10 @@ G_BEGIN_DECLS
  * Extensions are used (RFC4941).
  */
 typedef enum {
-	NM_SETTING_IP6_CONFIG_PRIVACY_UNKNOWN = -1,
-	NM_SETTING_IP6_CONFIG_PRIVACY_DISABLED = 0,
-	NM_SETTING_IP6_CONFIG_PRIVACY_PREFER_PUBLIC_ADDR = 1,
-	NM_SETTING_IP6_CONFIG_PRIVACY_PREFER_TEMP_ADDR = 2
+    NM_SETTING_IP6_CONFIG_PRIVACY_UNKNOWN            = -1,
+    NM_SETTING_IP6_CONFIG_PRIVACY_DISABLED           = 0,
+    NM_SETTING_IP6_CONFIG_PRIVACY_PREFER_PUBLIC_ADDR = 1,
+    NM_SETTING_IP6_CONFIG_PRIVACY_PREFER_TEMP_ADDR   = 2
 } NMSettingIP6ConfigPrivacy;
 
 /**
@@ -130,8 +135,8 @@ typedef enum {
  * Since: 1.2
  */
 typedef enum {
-	NM_SETTING_IP6_CONFIG_ADDR_GEN_MODE_EUI64 = 0,
-	NM_SETTING_IP6_CONFIG_ADDR_GEN_MODE_STABLE_PRIVACY = 1,
+    NM_SETTING_IP6_CONFIG_ADDR_GEN_MODE_EUI64          = 0,
+    NM_SETTING_IP6_CONFIG_ADDR_GEN_MODE_STABLE_PRIVACY = 1,
 } NMSettingIP6ConfigAddrGenMode;
 
 /**
@@ -140,29 +145,29 @@ typedef enum {
  * IPv6 Settings
  */
 struct _NMSettingIP6Config {
-	NMSettingIPConfig parent;
+    NMSettingIPConfig parent;
 };
 
 typedef struct {
-	NMSettingIPConfigClass parent;
+    NMSettingIPConfigClass parent;
 
-	/*< private >*/
-	gpointer padding[4];
+    /*< private >*/
+    gpointer padding[4];
 } NMSettingIP6ConfigClass;
 
-GType nm_setting_ip6_config_get_type (void);
+GType nm_setting_ip6_config_get_type(void);
 
-NMSetting *nm_setting_ip6_config_new (void);
+NMSetting *nm_setting_ip6_config_new(void);
 
-NMSettingIP6ConfigPrivacy nm_setting_ip6_config_get_ip6_privacy (NMSettingIP6Config *setting);
+NMSettingIP6ConfigPrivacy nm_setting_ip6_config_get_ip6_privacy(NMSettingIP6Config *setting);
 NM_AVAILABLE_IN_1_2
-NMSettingIP6ConfigAddrGenMode nm_setting_ip6_config_get_addr_gen_mode (NMSettingIP6Config *setting);
+NMSettingIP6ConfigAddrGenMode nm_setting_ip6_config_get_addr_gen_mode(NMSettingIP6Config *setting);
 NM_AVAILABLE_IN_1_4
-const char *nm_setting_ip6_config_get_token (NMSettingIP6Config *setting);
+const char *nm_setting_ip6_config_get_token(NMSettingIP6Config *setting);
 NM_AVAILABLE_IN_1_12
-const char *nm_setting_ip6_config_get_dhcp_duid (NMSettingIP6Config *setting);
-NM_AVAILABLE_IN_1_22_8
-gint32 nm_setting_ip6_config_get_ra_timeout (NMSettingIP6Config *setting);
+const char *nm_setting_ip6_config_get_dhcp_duid(NMSettingIP6Config *setting);
+NM_AVAILABLE_IN_1_24
+gint32 nm_setting_ip6_config_get_ra_timeout(NMSettingIP6Config *setting);
 
 G_END_DECLS
 
